/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
#ifndef GNUMERIC_EXPR_H
#define GNUMERIC_EXPR_H

#include "gnumeric.h"
#include "position.h"

/* Warning: if you add something here, see do_expr_as_string   ! */
/* Warning: if you add something here, see ms-formula-write.c  ! */
typedef enum {
	GNM_EXPR_OP_EQUAL,	/* Compare value equal */
	GNM_EXPR_OP_GT,		/* Compare value greather than  */
	GNM_EXPR_OP_LT,		/* Compare value less than */
	GNM_EXPR_OP_GTE,	/* Compare value greather or equal than */
	GNM_EXPR_OP_LTE,	/* Compare value less or equal than */
	GNM_EXPR_OP_NOT_EQUAL,	/* Compare for non equivalence */

	GNM_EXPR_OP_ADD,	/* Add  */
	GNM_EXPR_OP_SUB,	/* Subtract */
	GNM_EXPR_OP_MULT,	/* Multiply */
	GNM_EXPR_OP_DIV,	/* Divide */
	GNM_EXPR_OP_EXP,	/* Exponentiate */
	GNM_EXPR_OP_CAT,	/* String concatenation */

	GNM_EXPR_OP_FUNCALL,	/* Function call invocation */

	GNM_EXPR_OP_NAME,	/* Name reference */

        GNM_EXPR_OP_CONSTANT,	/* Constant value */
	GNM_EXPR_OP_CELLREF,	/* Cell content lookup (variable) */
	GNM_EXPR_OP_UNARY_NEG,	/* Sign inversion */
	GNM_EXPR_OP_UNARY_PLUS,	/* Mark as positive */
	GNM_EXPR_OP_PERCENTAGE,	/* Percentage (value/100) */
	GNM_EXPR_OP_ARRAY_CORNER,/* Top Corner of an array */
	GNM_EXPR_OP_ARRAY_ELEM,	/* General Array element */
	GNM_EXPR_OP_SET,	/* A set of expressions */
	GNM_EXPR_OP_RANGE_CTOR,	/* A constructed range eg A1:index(1,2) */
	GNM_EXPR_OP_INTERSECT	/* The intersection of multiple ranges */
} GnmExprOp;

/* Shorthands for case statements.  Easy to read, easy to maintain.  */
#define GNM_EXPR_OP_ANY_BINARY GNM_EXPR_OP_EQUAL: case GNM_EXPR_OP_GT: case GNM_EXPR_OP_LT: case GNM_EXPR_OP_GTE: \
	case GNM_EXPR_OP_LTE: case GNM_EXPR_OP_NOT_EQUAL: \
	case GNM_EXPR_OP_ADD: case GNM_EXPR_OP_SUB: case GNM_EXPR_OP_MULT: case GNM_EXPR_OP_DIV: \
	case GNM_EXPR_OP_EXP: case GNM_EXPR_OP_CAT
#define GNM_EXPR_OP_ANY_UNARY GNM_EXPR_OP_UNARY_NEG: case GNM_EXPR_OP_UNARY_PLUS : case GNM_EXPR_OP_PERCENTAGE

GnmExpr const *gnm_expr_new_constant	(GnmValue *v);
GnmExpr const *gnm_expr_new_unary	(GnmExprOp op, GnmExpr const *e);
GnmExpr const *gnm_expr_new_binary	(GnmExpr const *l, GnmExprOp op,
					 GnmExpr const *r);
GnmExpr const *gnm_expr_new_funcall	(GnmFunc *func,
					 GnmExprList *args);
GnmExpr const *gnm_expr_new_funcall1	(GnmFunc *func,
					 GnmExpr const *arg0);
GnmExpr const *gnm_expr_new_funcall2	(GnmFunc *func,
					 GnmExpr const *arg0,
					 GnmExpr const *arg1);
GnmExpr const *gnm_expr_new_funcall3	(GnmFunc *func,
					 GnmExpr const *arg0,
					 GnmExpr const *arg1,
					 GnmExpr const *arg2);
GnmExpr const *gnm_expr_new_name	(GnmNamedExpr *name,
					 Sheet *sheet_scope, Workbook *wb_scope);
GnmExpr const *gnm_expr_new_cellref	(GnmCellRef const *cr);
GnmExpr const *gnm_expr_new_set		(GnmExprList *args);

GnmValue      *gnm_expr_get_range    (GnmExpr const *expr);
GnmFunc       *gnm_expr_get_func_def (GnmExpr const *expr);

void	  gnm_expr_free              (GnmExpr const *expr);
GnmExpr const *gnm_expr_copy         (GnmExpr const *expr);
gboolean  gnm_expr_is_rangeref 	     (GnmExpr const *expr);
gboolean  gnm_expr_is_data_table     (GnmExpr const *expr,
				      GnmCellPos *c_in, GnmCellPos *r_in);
char	 *gnm_expr_as_string	     (GnmExpr const *expr, GnmParsePos const *pp,
				      GnmExprConventions const *fmt);
gboolean  gnm_expr_contains_subtotal (GnmExpr const *expr);

struct _GnmExprRelocateInfo {
	GnmEvalPos pos;

	GnmRange   origin;	    /* References to cells in origin_sheet!range */
	Sheet     *origin_sheet;    /* should to adjusted */
	Sheet     *target_sheet;    /* to point at this sheet */
	int col_offset, row_offset; /* and offset by this amount */
	enum {
		GNM_EXPR_RELOCATE_STD,	/* do standard relocation */
		GNM_EXPR_RELOCATE_COLS, /* ins/del col */
		GNM_EXPR_RELOCATE_ROWS	/* ins/del row */
	} reloc_type;
};

struct _GnmExprRewriteInfo {
	enum {
		/* invalidate references to specified sheet */
		GNM_EXPR_REWRITE_INVALIDATE_SHEETS,
		/* relocate only fully absolute references */
		GNM_EXPR_REWRITE_NAME,
		/* relocate all references */
		GNM_EXPR_REWRITE_EXPR
	} rw_type;

	union {
		struct {
			int dummy;
		} sheets;
		GnmExprRelocateInfo relocate;
	} u;
};

GnmValue *gnm_expr_eval (GnmExpr const *expr, GnmEvalPos const *pos,
			 GnmExprEvalFlags flags);

/*****************************************************************************/

#define gnm_expr_list_append(l,e)  g_slist_append ((l), (gpointer)(e))
#define gnm_expr_list_prepend(l,e) g_slist_prepend ((l), (gpointer)(e))
#define gnm_expr_list_length(l)	   g_slist_length((GSList *)(l)) /* const cast */
#define gnm_expr_list_free	   g_slist_free
void 	 gnm_expr_list_unref	  (GnmExprList *list);

/*****************************************************************************/

#define IS_GNM_EXPR_TOP(et) ((et) && (et)->magic == GNM_EXPR_TOP_MAGIC)

struct _GnmExprTop {
	guint32 magic;
	guint32 refcount;
	GnmExpr const *expr;
};

GnmExprTop const *gnm_expr_top_new (GnmExpr const *e);
GnmExprTop const *gnm_expr_top_new_constant (GnmValue *v);
void gnm_expr_top_ref (GnmExprTop const *texpr);
void gnm_expr_top_unref (GnmExprTop const *texpr);

GnmExprTop const *gnm_expr_top_new_array_corner(int cols, int rows, GnmExpr const *expr);
GnmExprTop const *gnm_expr_top_new_array_elem  (int x, int y);

gboolean gnm_expr_top_is_shared (GnmExprTop const *texpr);
gboolean gnm_expr_top_is_err (GnmExprTop const *texpr, GnmStdError e);
gboolean  gnm_expr_top_is_rangeref (GnmExprTop const *texpr);
GnmValue const*gnm_expr_top_get_constant (GnmExprTop const *texpr);
gboolean gnm_expr_top_contains_subtotal (GnmExprTop const *texpr);
GSList *gnm_expr_top_referenced_sheets (GnmExprTop const *texpr);
GnmExpr const *gnm_expr_top_first_funcall (GnmExprTop const *texpr);
void gnm_expr_top_get_boundingbox (GnmExprTop const *texpr, GnmRange *bound);

GnmExprTop const *gnm_expr_top_rewrite (GnmExprTop const *texpr,
					GnmExprRewriteInfo const *rwinfo);
GSList *gnm_expr_top_get_ranges (GnmExprTop const *texpr);
gboolean gnm_expr_top_equal (GnmExprTop const *te1, GnmExprTop const *te2);
char *gnm_expr_top_as_string (GnmExprTop const *texpr,
			      GnmParsePos const *pp,
			      GnmExprConventions const *fmt);
void gnm_expr_top_as_gstring (GString *target,
			      GnmExprTop const *texpr,
			      GnmParsePos const *pp,
			      GnmExprConventions const *fmt);
GnmValue *gnm_expr_top_eval (GnmExprTop const *texpr,
			     GnmEvalPos const *pos,
			     GnmExprEvalFlags flags);
GnmValue      *gnm_expr_top_get_range    (GnmExprTop const *texpr);

GnmExprArrayCorner const *gnm_expr_top_get_array_corner (GnmExprTop const *texpr);
gboolean gnm_expr_top_is_array_elem (GnmExprTop const *texpr);

/*****************************************************************************/

typedef struct {
	GHashTable *exprs, *ptrs;

	int nodes_in, nodes_stored;
} ExprTreeSharer;

ExprTreeSharer *expr_tree_sharer_new     (void);
void            expr_tree_sharer_destroy (ExprTreeSharer *);
GnmExpr const  *expr_tree_sharer_share   (ExprTreeSharer *, GnmExpr const *expr);

void expr_init (void);
void expr_shutdown (void);

#endif /* GNUMERIC_EXPR_H */
